/******************************************************************************
*
*  MultiTool Sun 3/4 Standalone	PARSYTEC BBK-V1/V2 VMTM Link Adaptor Driver
*
*  File : "interf.c"
*
*  Version 1.04 from 31.7.1989
*
*  Date : 890731
*
*  Copyright PARSYTEC 1988, 1989
*
******************************************************************************/

/* include files */
#include "return.h"
#include <stdio.h>


/* link driver variables */
static link_id = -1;	/* id of link-adapter in use */
int terrno = 0;		/* global error-code (see 'vmeboard.h') */



int link_init (board_type, vme_addr)

/* ***  Warning: Both parameters are completely ignored !
 */
int	 board_type;
unsigned long vme_addr;		/* base address of transputer board */

/* This function is supplied for compatibility reasons only.
 * Using the link-interface does NOT allow you to specifiy the VME-bus
 * address (for security reasons).
 *
 */

{   int lerror;		/* local error code */;

    lerror = OpenLink ("");	/* use any free link */
    if ( lerror < 0 ) {
	/* cannot open the link-adaptor */
	terrno = lerror;
	return (T_INIT);
    } else {
	/* link successfully opened
	 * store the returned link-id for
	 * later use
	 */
	link_id = lerror;
	return (T_OK);
    }
}



int link_output (byte_count, data_address, timeout)

int   byte_count;	/* number of bytes to transfer */
char *data_address;	/* start of data transfer area */
int   timeout;		/* timeout period in 0.1 seconds */

/* Write a sequence of bytes to the link adaptor within a specified time.
 * Return the number of bytes successfully output.
 *
 * Global variables used:
 *
 *	link_id;	/* id of link_adaptor * /
 *	terrno;		/* global error-code for transputer interface * /
 */
{   int lerror;		/* local error code */

    lerror = WriteLink (link_id, data_address, byte_count, timeout);
    if ( lerror < 0 ) {
	/* some kind of error has happened */
	terrno = lerror;
	return (T_WRITE);
    }
    /* return number of send bytes */
    return (lerror);
}



int link_input (byte_count, data_address, timeout)

int   byte_count;	/* number of bytes to transfer */
char *data_address;	/* start of data transfer area */
int   timeout;		/* timeout period in 0.1 seconds */

/* Read a sequence of bytes from the link adaptor within a specified time.
 * Return the number of bytes successfully input.
 *
 * Global variables used:
 *
 *	link_id;	/* id of link_adaptor * /
 *	terrno;		/* global error-code for transputer interface * /
 */
{   int lerror;		/* local error code */

    lerror = ReadLink (link_id, data_address, byte_count, timeout);
    if ( lerror < 0 ) {
	/* some kind of error has happened */
	terrno = lerror;
	return (T_READ);
    }
    /* return number of received bytes */
    return (lerror);
}



int code_load (boot_file_name, timeout)

char *boot_file_name;
int   timeout;

/* Reset the transputer, open the specified file for reading
 * and output its contents to the transputer.
 *
 * Global variables:
 *
 *	link_id		contains the id of the link to use.
 *			It must be opened before by 'OpenLink' (link-interface)
 *
 * Return values:
 *
 *	T_OK		Everything is ok
 *	T_RESET		cannot reset transputer ('terrno' set)
 *	T_TIMEOUT	timeout during code download
 *	T_FILE_OPEN	cannot open file ('errno' set)
 *	T_FILE_READ	error during file read ('errno' set)
 *	T_FILE_CLOSE	cannot close file ('errno' set)
 */
 
#define BOOT_BUFFER_LENGTH (1024 * 4)

{   FILE *boot_file;	/* file descriptor of boot file */
    register int count = 0;
    int lerror;		/* local error-code */
    char file_buffer[BOOT_BUFFER_LENGTH];

    if ( (lerror = ResetLink (link_id)) < 0) {
	/* failed to reset transputer */
	terrno = lerror;
	return (T_RESET);
    }
    if ( (boot_file = fopen (boot_file_name, "r")) != NULL ) {
        while (1) {
            count = fread ( file_buffer, 1, sizeof (file_buffer), boot_file );
            if (count > 0) {
		/* send file_buffer through link with timeout */
		lerror = WriteLink (link_id, file_buffer, count, timeout);
		if (count != lerror) {
		    /* error while sending to transputer */
		    if ( lerror < 0 ) {
			/* serious error */
			terrno = lerror;
			return (T_SEND);
		    } else {
			/* timeout during sending */
			return (T_TIMEOUT);
		    }
		} /* if send ok */
            } else {
		/* no more data from bootfile */
                if ( feof (boot_file) ) {
		    /* end of file */
                    if ( fclose (boot_file) )
			return (T_FILE_CLOSE);
		    else {
			return (T_OK);
		    }
                } else {
		    /* error during read */
		    return (T_FILE_READ);
		}
            }
        } /* while TRUE */
    } else {
	return (T_FILE_OPEN);
    }
}
