/******************************************************************************/
/* SCSI_BRIDGE.H - library functions for using of the device driver           */
/******************************************************************************/
/*                                                                            */
/* AUTHOR:    Christoph Steinbrueck, 04/05/1994                               */
/* COPYRIGHT: Parsytec Computer GmbH                                          */
/* CHANGED:                                                                   */
/* SccsId     @(#)scsi_bridge.h	1.1	94/12/08                        */
/* ROLE:      contains neccessary functions for using tape driver/SCSI-Bridge */
/*                                                                            */
/******************************************************************************/
/******************************************************************************/

#define CODE
#define ENDCODE

#ifdef INFO

NODE Defines for SCSI commands, SCSI-Loop General Commands, SCSI-Loop General Description, SCSI-Loop
@section Defines for SCSI commands

Here are the defines used between SCSI-Loop and Host.

@menu
* struct link_state::
* defines for link_state::
@end menu

#endif

#ifdef	SCSI
extern int      ResetDevice ();		/* reset a link (slot only) */
extern int      SCSI_Get_State ();	/* non blocking */
extern int	SCSI_rewind();		/* for tape driver */
#endif

#ifdef INFO
NODE struct link_state, defines for link_state, , Defines for SCSI commands
@subsection struct link_state

This structure is returned by calling SCSI_GetState().

#endif
CODE
typedef struct link_state {
  unsigned int    slot_size;
  unsigned int    state;
  /* Bit 0-15 channel number of next channel for reading  */
  /* Bit 16 is 0 if no slot for reading                   */
  /* Bit 17 is not 0 if no slot for writing               */
  /* Bit 18 reflects EOM for write                        */
  /* Bit 19 reflects EOM for read                         */
  /* Bit 20 is set if timeout occured                     */
  /* Bit 21 is set if an error occured                    */
  /* Bit 22 is set if there are any not empty write slots */
  unsigned int    link_status;
  unsigned int    SlotSize;
  unsigned int    error_buffer;
}               link_state_t;
ENDCODE

#ifdef INFO

see also: @ref\{defines for link_state\}.
#endif

#ifdef INFO
NODE defines for link_state, , struct link_state, Defines for SCSI commands
@subsection defines for link_state

This are the defines used for link_state.state and link_state.link_status.

#endif

CODE
#define READ_STATE		0x00010000	/* bit mask for states */
#define WRITE_STATE		0x00020000
#define WRITE_EOM_STATE		0x00040000
#define READ_EOM_STATE		0x00080000
#define SELECT_TIMEOUT		0x00100000
#define ERROR_OCCURED		0x00200000
#define WRITE_SLOTS_NOT_EMPTY	0x00400000

#define CHANNEL_NUMBER_MSK	0x0000FFFF

#define LINK_STATUS_BUSY                 0
#define LINK_STATUS_RESET                1

ENDCODE

#ifdef INFO

see also: @ref\{struct link_state\}.
#endif

/* !!! link_status are soon obsolete 'cause of link_state.link_status !!! */
typedef struct link_status {		/* status of physical link and it's
					 * process */
  unsigned int    link_error;
}               link_status_t;

#ifdef GARBAGE
#define SCSI_CMD_WRITE			0
#define SCSI_CMD_WRITE_EOM		1	/* write EOM Slot */
#define SCSI_CMD_READ			2
#endif

/*
 * Commands between DServer and SCSI-T8-bridge
 */

#define SCSI_CMD_SET_PX_HDR		0
#define SCSI_CMD_GET_STATE		1
#define SCSI_CMD_GET_VERSION		2
#define SCSI_CMD_BURN_EPROM		3
#define SCSI_CMD_SET_LINK_MODE		4
#define SCSI_CMD_RESET_LINK		5
#define SCSI_CMD_RESET_DEVICE		6
#define SCSI_CMD_GET_STATUS		7
#define SCSI_CMD_GET_DUMMY		8	/* clear one Slot */
#define SCSI_CMD_DISABLE_LINKS		9
#define SCSI_CMD_SYNC			11
#define SCSI_CMD_RESETED		12
#define SCSI_CMD_MONITOR		13
#define SCSI_CMD_TESTREAD		14
#define SCSI_CMD_BURNFLASH              15      /* burn new EPROM version */
#define SCSI_CMD_READVERSION		16

#define SCSI_CMD_SHIFT			16	/* shift size for Commands */

/*
 * States for DServer and SCSI-T8-bridge
 */

#define STATE_FREE	       -1	   /* read or write data */
#define STATE_FREE_PACKET      -2

#define STATE_SELECT_READ      -4
#define STATE_WAIT_SELECT_READ -5
#define STATE_DISABLE_LINKS    -6
/* stream mode */
#define STATE_WRITE_TIMEOUT    -7 /* s- mode, wait for write cmd (timeout)*/
#define STATE_WRITELINK	       -8 /* s- mode, wait for write cmd (data) */
#define STATE_WAIT_WRITELINK   -9 /* s- mode, wait for OutServer ready */
#define STATE_WRITTEN_NUMBER   -10 /* s- mode, wait for read cmd (byte count) */

#define STATE_READSTATE	       -11   /* s- mode, wait for read cmd (1 byte) */
#define STATE_WAIT_READSTATE   -12   /* s- mode, wait for InServer (Select) */
#define STATE_READLINK         -13   /* s- mode, wait for read cmd (data) */
#define STATE_WAIT_READLINK    -14   /* s- mode, wait for InServer (data) */
#define STATE_TESTREAD         -15   /* s- mode, wait for read cmd (1 byte) */
#define STATE_WAIT_TESTREAD    -16   /* s- mode, wait for InServer (Select) */

#define STATE_WRITE_EPROM	-17  /* next write sends new EPROM version */
#define STATE_READ_EPROM_OK     -18  /* next read ask, whether writing to EPROM was ok - ok */
#define STATE_READ_EPROM_FAIL   -19  /* next read ask, whether writing to EPROM was ok - fail */
#define STATE_READ_VERSION     -20  /* next read gets version string */
#define LINK_MODE_PACKET		1
#define LINK_MODE_STREAM		0

#define MAX_LINKS			4

/*
 * Errors SCSI-T8-bridge
 */

#define	SCSI_ERR_NO_ERROR		(int)0x00
#define	SCSI_ERR_PX_HDR			(int)0x01
#define	SCSI_ERR_CHAN_IN		(int)0x02
#define	SCSI_ERR_CHAN_OUT		(int)0x04
#define	SCSI_ERR_BODY_SIZE		(int)0x08
#define	SCSI_ERR_PROC_ALLOC		(int)0x10
#define	SCSI_ERR_OUT_TIMEOUT		(int)0x20
#define	SCSI_ERR_BURN_EPROM		(int)0x40

/*
 * Defines for Slot Caching
 */

typedef struct {
	int     channel_number;
	int     msg_size;
} SCSI_MsgHead_t;

#define	SCSI_MSG_HEADSIZE		sizeof(SCSI_MsgHead_t)

/*
 * minor structure used by SCSI_Loop and IN-Server
 */

typedef struct min_struct {
  int             read_state;
  int             write_state;
  int             link_mode;
  int             read_slot;
  int             write_slot;
  int             rcount;
  int             wcount;
  int             wlen;
  int             write_failed;
  int             write_c;
  int             read_c;
  int             space;
  char            *sense_data;
  int             old_read_state;
  int             old_write_state;
}               min_struct_t;

